@@ -60,9 +60,9 @@ fi
 VIRTUALENVWRAPPER_ENV_BIN_DIR="bin"
 if [ "$OS" = "Windows_NT" ] && [ "$MSYSTEM" = "MINGW32" ]
 then
-	# Only assign this for msys, cygwin use standard Unix paths
-	# and its own python installation
-	VIRTUALENVWRAPPER_ENV_BIN_DIR="Scripts"
+    # Only assign this for msys, cygwin use standard Unix paths
+    # and its own python installation
+    VIRTUALENVWRAPPER_ENV_BIN_DIR="Scripts"
 fi
 
 # Let the user override the name of the file that holds the project
@@ -72,6 +72,15 @@ then
     export VIRTUALENVWRAPPER_PROJECT_FILENAME=".project"
 fi
 
+function virtualenvwrapper_expandpath {
+        if [ "$1" = "" ]; then
+            return 1
+        else
+            "$VIRTUALENVWRAPPER_PYTHON" -c "import os,sys; sys.stdout.write(os.path.realpath(os.path.expanduser(os.path.expandvars(\"$1\")))+'\n')"
+            return 0
+        fi
+}
+
 function virtualenvwrapper_derive_workon_home {
     typeset workon_home_dir="$WORKON_HOME"
 
@@ -99,7 +108,7 @@ function virtualenvwrapper_derive_workon_home {
         # - Removing extra slashes (e.g., when TMPDIR ends in a slash)
         # - Expanding variables (e.g., $foo)
         # - Converting ~s to complete paths (e.g., ~/ to /home/brian/ and ~arthur to /home/arthur)
-        workon_home_dir=$("$VIRTUALENVWRAPPER_PYTHON" -c "import os,sys; sys.stdout.write(os.path.expandvars(os.path.expanduser(\"$workon_home_dir\"))+'\n')")
+        workon_home_dir=$(virtualenvwrapper_expandpath "$workon_home_dir")
     fi
 
     echo "$workon_home_dir"
@@ -118,7 +127,7 @@ function virtualenvwrapper_verify_workon_home {
         then
             echo "NOTE: Virtual environments directory $WORKON_HOME does not exist. Creating..." 1>&2
         fi
-        mkdir -p $WORKON_HOME
+        mkdir -p "$WORKON_HOME"
         RC=$?
     fi
     return $RC
@@ -130,28 +139,29 @@ function virtualenvwrapper_verify_workon_home {
 function virtualenvwrapper_tempfile {
     # Note: the 'X's must come last
     typeset suffix=${1:-hook}
-    typeset file="`\mktemp -t virtualenvwrapper-$suffix-XXXXXXXXXX`"
-    if [ $? -ne 0 ]
+    typeset file
+
+    file="`\mktemp -t virtualenvwrapper-$suffix-XXXXXXXXXX`"
+    if [ $? -ne 0 ] || [ -z "$file" ] || [ ! -f "$file" ]
     then
         echo "ERROR: virtualenvwrapper could not create a temporary file name." 1>&2
         return 1
     fi
-    trap "\rm -f '$file' >/dev/null 2>&1" EXIT
     echo $file
     return 0
 }
 
 # Run the hooks
 function virtualenvwrapper_run_hook {
-    typeset hook_script="$(virtualenvwrapper_tempfile ${1}-hook)"
-    if [ -z "$hook_script" ]
-    then
-        echo "ERROR: Could not create temporary file name. Make sure TMPDIR is set." 1>&2
-        return 1
-    fi
+    typeset hook_script
+    typeset result
+
+    hook_script="$(virtualenvwrapper_tempfile ${1}-hook)" || return 1
+
     if [ -z "$VIRTUALENVWRAPPER_LOG_DIR" ]
     then
         echo "ERROR: VIRTUALENVWRAPPER_LOG_DIR is not set." 1>&2
+        \rm -f "$hook_script"
         return 1
     fi
     "$VIRTUALENVWRAPPER_PYTHON" -c 'from virtualenvwrapper.hook_loader import main; main()' $HOOK_VERBOSE_OPTION --script "$hook_script" "$@"
@@ -162,12 +172,13 @@ function virtualenvwrapper_run_hook {
         if [ ! -f "$hook_script" ]
         then
             echo "ERROR: virtualenvwrapper_run_hook could not find temporary file $hook_script" 1>&2
+            \rm -f "$hook_script"
             return 2
         fi
         # cat "$hook_script"
         source "$hook_script"
     fi
-    \rm -f "$hook_script" >/dev/null 2>&1
+    \rm -f "$hook_script"
     return $result
 }
 
@@ -298,7 +309,7 @@ function mkvirtualenv_help {
     echo;
     echo 'virtualenv help:';
     echo;
-    virtualenv -h;
+    virtualenv $@;
 }
 
 # Create a new environment, in the WORKON_HOME.
@@ -334,15 +345,16 @@ function mkvirtualenv {
             -a)
                 i=$(( $i + 1 ));
                 project="${in_args[$i]}";;
-            -h)
-                mkvirtualenv_help;
+            -h|--help)
+                mkvirtualenv_help $a;
                 return;;
             -i)
                 i=$(( $i + 1 ));
                 packages="$packages ${in_args[$i]}";;
             -r)
                 i=$(( $i + 1 ));
-                requirements="${in_args[$i]}";;
+                requirements="${in_args[$i]}";
+                requirements=$(virtualenvwrapper_expandpath "$requirements");;
             *)
                 if [ ${#out_args} -gt 0 ]
                 then
@@ -374,8 +386,8 @@ function mkvirtualenv {
     # we should switch to the environment and run the hook.
     [ ! -d "$WORKON_HOME/$envname" ] && return 0
 
-	# If they gave us a project directory, set it up now
-	# so the activate hooks can find it.
+    # If they gave us a project directory, set it up now
+    # so the activate hooks can find it.
     if [ ! -z "$project" ]
     then
         setvirtualenvproject "$WORKON_HOME/$envname" "$project"
@@ -462,38 +474,38 @@ function lsvirtualenv {
     typeset long_mode=true
     if command -v "getopts" &> /dev/null
     then
-		# Use getopts when possible
-    	OPTIND=1
-		while getopts ":blh" opt "$@"
-		do
-			case "$opt" in
-				l) long_mode=true;;
-				b) long_mode=false;;
-				h)  _lsvirtualenv_usage;
-					return 1;;
-				?) echo "Invalid option: -$OPTARG" >&2;
-					_lsvirtualenv_usage;
-					return 1;;
-			esac
-		done
+        # Use getopts when possible
+        OPTIND=1
+        while getopts ":blh" opt "$@"
+        do
+            case "$opt" in
+                l) long_mode=true;;
+                b) long_mode=false;;
+                h)  _lsvirtualenv_usage;
+                    return 1;;
+                ?) echo "Invalid option: -$OPTARG" >&2;
+                    _lsvirtualenv_usage;
+                    return 1;;
+            esac
+        done
     else
-    	# fallback on getopt for other shell
-	    typeset -a args
-	    args=($(getopt blh "$@"))
-	    if [ $? != 0 ]
-	    then
-	        _lsvirtualenv_usage
-	        return 1
-	    fi
-	    for opt in $args
-	    do
-	        case "$opt" in
-	            -l) long_mode=true;;
-	            -b) long_mode=false;;
-	            -h) _lsvirtualenv_usage;
-	                return 1;;
-	        esac
-	    done
+        # fallback on getopt for other shell
+        typeset -a args
+        args=($(getopt blh "$@"))
+        if [ $? != 0 ]
+        then
+            _lsvirtualenv_usage
+            return 1
+        fi
+        for opt in $args
+        do
+            case "$opt" in
+                -l) long_mode=true;;
+                -b) long_mode=false;;
+                -h) _lsvirtualenv_usage;
+                    return 1;;
+            esac
+        done
     fi
 
     if $long_mode
@@ -532,8 +544,8 @@ function showvirtualenv {
 # Usage: workon [environment_name]
 #
 function workon {
-	typeset env_name="$1"
-	if [ "$env_name" = "" ]
+    typeset env_name="$1"
+    if [ "$env_name" = "" ]
     then
         lsvirtualenv -b
         return 1
@@ -594,7 +606,7 @@ function workon {
 
     virtualenvwrapper_run_hook "post_activate"
 
-	return 0
+    return 0
 }
 
 
@@ -609,7 +621,7 @@ function virtualenvwrapper_get_python_version {
 
 # Prints the path to the site-packages directory for the current environment.
 function virtualenvwrapper_get_site_packages_dir {
-    "$VIRTUAL_ENV/bin/python" -c "import distutils; print(distutils.sysconfig.get_python_lib())"
+    "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_ENV_BIN_DIR/python" -c "import distutils; print(distutils.sysconfig.get_python_lib())"
 }
 
 # Path management for packages outside of the virtual env.
@@ -707,7 +719,7 @@ function lssitepackages {
     virtualenvwrapper_verify_workon_home || return 1
     virtualenvwrapper_verify_active_environment || return 1
     typeset site_packages="`virtualenvwrapper_get_site_packages_dir`"
-    ls $@ $site_packages
+    ls $@ "$site_packages"
 
     path_file="$site_packages/_virtualenv_path_extensions.pth"
     if [ -f "$path_file" ]
@@ -897,7 +909,7 @@ function mkproject {
 
     cd "$PROJECT_HOME"
 
-    virtualenvwrapper_run_hook project.pre_mkproject $envname
+    virtualenvwrapper_run_hook "project.pre_mkproject" $envname
 
     echo "Creating $PROJECT_HOME/$envname"
     mkdir -p "$PROJECT_HOME/$envname"
@@ -912,10 +924,10 @@ function mkproject {
         # For some reason zsh insists on prefixing the template
         # names with a space, so strip them out before passing
         # the value to the hook loader.
-        virtualenvwrapper_run_hook --name $(echo $t | sed 's/^ //') project.template $envname
+        virtualenvwrapper_run_hook --name $(echo $t | sed 's/^ //') "project.template" $envname
     done
 
-    virtualenvwrapper_run_hook project.post_mkproject
+    virtualenvwrapper_run_hook "project.post_mkproject"
 }
 
 # Change directory to the active project
